package ru.ulstu.fc.project.controller;

import io.swagger.v3.oas.annotations.Hidden;
import org.springframework.security.access.annotation.Secured;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import ru.ulstu.fc.project.model.RunProjectForm;
import ru.ulstu.fc.project.service.ProjectService;
import ru.ulstu.fc.project.service.ProjectVariableService;
import ru.ulstu.fc.rule.service.FuzzyInferenceService;
import ru.ulstu.fc.user.model.UserRoleConstants;

@Controller
@Hidden
@RequestMapping("runProject")
@Secured({UserRoleConstants.ADMIN})
public class ProjectRunController {
    private final ProjectService projectService;
    private final FuzzyInferenceService fuzzyInferenceService;
    private final ProjectVariableService projectVariableService;

    public ProjectRunController(ProjectService projectService,
                                FuzzyInferenceService fuzzyInferenceService,
                                ProjectVariableService projectVariableService) {
        this.projectService = projectService;
        this.fuzzyInferenceService = fuzzyInferenceService;
        this.projectVariableService = projectVariableService;
    }

    @GetMapping("init/{projectId}")
    public String getProjects(@PathVariable(value = "projectId") Integer projectId, Model model) {
        model.addAttribute("project", projectService.getById(projectId));
        model.addAttribute("runProjectForm", new RunProjectForm(projectId));
        model.addAttribute("variables", projectVariableService.getInputByProjectId(projectId));
        return "project/init";
    }

    @PostMapping("run")
    public String run(RunProjectForm runProjectForm, Model model) {
        model.addAttribute("projectId", runProjectForm.getProjectId());
        model.addAttribute("response",
                fuzzyInferenceService.getProjectFuzzyInference(
                        runProjectForm.getProjectId(),
                        runProjectForm.getVariableValues()));
        return "project/result";
    }
}
